#ifndef bqtInstrumentHH
#define bqtInstrumentHH

#include <deque>
#include <vector>

#include "snd.hh"

struct sampleitem
{
    int duty; /* Duty at this row */
    /* Frequency changes to the original */
    int wavelenadd;
    long wavelenmul;
    
    /* Volume changes to the original */
    int voladd;
    long volmul;
};

struct sample
{
    SNDChannel::chantype type;
    unsigned beginrow;
    unsigned basewavelen;
    unsigned char basevol;
    unsigned char defaultvol;
    unsigned char midiins;

    // Instrumentista on tss 7 elementin
    // alkuptk ja 7 elementin loppuptk.
    std::deque<sampleitem> alkutree;
    /*
    std::deque<sampleitem> lopputree;
    */
    
    sample() : beginrow(0),basewavelen(0),basevol(0),defaultvol(0) {}
    
    enum comparetype { undefined=0, add, mul };
    /* Whether the wavelen is compared with multiplication or addition */
    comparetype wavelentype;
    /* Whether the volume is compared with multiplication or addition */
    comparetype voltype;

    /* For noise channels, duty doesn't count */
    /* For triangle channels, duty and volume don't count */
    
    const bool dutytest() const { return type == SNDChannel::sqr; }
    const bool volutest() const { return type != SNDChannel::tri; }
    const bool wavetest() const { return type != SNDChannel::sqr; }
};

struct SampleMatchType
{
    unsigned length;
    bool waveaddmatch, wavemulmatch;
    bool voladdmatch, volmulmatch;
    
    SampleMatchType() : length(0),
      waveaddmatch(true), wavemulmatch(true),
      voladdmatch(true), volmulmatch(true) { }
    SampleMatchType(bool) : length(0) { }
};

extern std::vector<sample> repository;

extern void DumpSampleTree(const sample &ins, const std::deque<sampleitem> &tree);
extern void DumpSample(const sample &ins);

extern unsigned AnalyzeSample(const sample &smp);

extern void GuessMIDIfor(sample &smp);

#endif
